# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import json
from unittest import TestCase

import mock
from faker import Faker

from notification_service.push.AWS import AWSMessage, AWSSender, AWSiOSContentKey
from notification_service.push.base import ClientSystem, ServiceProvider, Environment
from tests.unit.push import TestMessage, create_Message

faker = Faker()


def create_AWSMessage(**kwargs):
    return create_Message(AWSMessage, **kwargs)


class test_AWSMessage(TestMessage):

    def test_android(self):
        msg = create_AWSMessage(client_type='aws-android',
                                title=self.title, content=self.content)
        assert msg.client_system == ClientSystem.Android
        assert msg.service_provider == ServiceProvider.AWS

    def test_android_custom_msg(self):
        msg = create_AWSMessage(client_type='aws-android',
                                title=self.title, content=self.content, custom_msg={
                'GCM': {
                    'data': {
                        'title': self.new_title,
                        'message': self.new_content,
                    }
                }
            })
        data = msg.to_msg_object()
        assert json.loads(data['GCM'])['data']['title'] == self.new_title
        assert json.loads(data['GCM'])['data']['message'] == self.new_content

    def test_ios(self):
        msg = create_AWSMessage(client_type='aws-ios', title=self.title,
                                content=self.content,
                                environment=Environment.production)
        assert msg.client_system == ClientSystem.iOS
        assert msg.service_provider == ServiceProvider.AWS

    def test_ios_custom_msg(self):
        environment = Environment.production
        ios_content_key = AWSiOSContentKey.__members__[environment.value].value
        msg = create_AWSMessage(client_type='aws-ios', title=self.title,
                                content=self.content,
                                environment=environment, custom_msg={
                ios_content_key: {
                    'aps': {
                        'alertTitle': self.new_title,
                        'alert': self.new_content,
                    }
                }
            })
        data = msg.to_msg_object()
        assert json.loads(data[ios_content_key])[
                   'aps']['alertTitle'] == self.new_title
        assert json.loads(data[ios_content_key])[
                   'aps']['alert'] == self.new_content


class test_AWSSender(TestCase):

    def setUp(self):
        self.target = 'arn:aws:sns:us-east-1:108935729014:endpoint/GCM/D3_test\
            /ec332e06-06d4-3457-b2fc-68c8f543aa14'
        self.application_arn = 'arn:aws:sns:us-east-1:108935729014:app/GCM/D3_test'
        self.region_name = 'us-east-1'
        self.access_id = 'AKIAJ5NDSRIBIZ6O5VEQ'
        self.secret_key = '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'
        self.mock_publish_success = mock.Mock(return_value={'ResponseMetadata': {
            'RetryAttempts': 0, 'HTTPStatusCode': 200,
            'RequestId': '50d68dd3-5b5f-5db7-b4c3-d95ebcc3438e',
            'HTTPHeaders': {'x-amzn-requestid': '50d68dd3-5b5f-5db7-b4c3-d95ebcc3438e',
                            'date': 'Fri, 14 Dec 2018 17:16:12 GMT',
                            'content-length': '294',
                            'content-type': 'text/xml'}},
            u'MessageId': 'c4c1c592-7bfe-5c1c-bda6-911f9c627225'})
        self.client_types = ['aws-ios', 'aws-android']

    def test_sender_with_application_arn(self):
        for client_type in self.client_types:
            self._test_sender_with_application_arn(client_type)

    def test_sender_with_region_name(self):
        for client_type in self.client_types:
            self._test_sender_with_region_name(client_type)

    def _test_sender_with_application_arn(self, client_type):
        msg = create_AWSMessage(client_type=client_type)
        sender = AWSSender(access_id=self.access_id, secret_key=self.secret_key,
                           application_arn=self.application_arn,
                           client_type=client_type)
        assert sender.region_name != ''

        sender._client.publish = self.mock_publish_success
        assert sender.push_single_device(msg, self.target).is_success

    def _test_sender_with_region_name(self, client_type):
        msg = create_AWSMessage(client_type=client_type)
        sender = AWSSender(access_id=self.access_id, secret_key=self.secret_key,
                           region_name=self.region_name, client_type=client_type)
        assert sender.region_name != ''

        sender._client.publish = self.mock_publish_success
        assert sender.push_single_device(msg, self.target).is_success

    def test_check_authorization(self):
        for client_type in self.client_types:
            sender = AWSSender(access_id=self.access_id, secret_key=self.secret_key,
                               application_arn=self.application_arn,
                               client_type=client_type)
            assert sender.check_authorization().is_success

    def test_create_platform_endpoint(self):
        for client_type in self.client_types:
            sender = AWSSender(access_id=self.access_id, secret_key=self.secret_key,
                               application_arn=self.application_arn,
                               client_type=client_type)
            token = 'c7ef3cb5e31a40a084672d55f46cfd964b9b32ddf62d3b4b0609c4a52837aa41'
            result = sender.create_platform_endpoint(token)
            assert result.is_success



# -*- coding: utf-8 -*-
from __future__ import absolute_import

import logging

import jpush
from enum import Enum

from notification_service.push.base import (Sender, Message, ClientSystem,
                                            ServiceProvider, PushType, Environment)
from notification_service.exceptions import ClientTypeError, SenderError, CidTypeError
from notification_service.result import PushResult
from notification_service.states import (
    UnknowPushState, PushExcetion, Success)
from notification_service.utils import update_sub_dict

logger = logging.getLogger(__name__)


class CidType(Enum):
    alias = 1
    registration_id = 2


class JiguangMessage(Message):

    def _to_msg_object_iOS(self):
        msg = {
            'alert': {
                'title': self.title,
                'body': self.content,
            }
        }
        msg = self._apply_push_type(self._apply_custom_msg(msg))
        try:
            return jpush.ios(**msg)
        except Exception, e:
            logger.warning(
                'jiguang iOS push apply custom_msg failure. error msg: {}'.format(e))
            return jpush.android(self.content, self.title)

    def _to_msg_object_android(self):
        msg = {
            'alert': self.content,
            'title': self.title,
        }
        msg = self._apply_push_type(self._apply_custom_msg(msg))
        try:
            return jpush.android(**msg)
        except Exception, e:
            logger.warning(
                'jiguang android push apply custom_msg failure. error msg: {}'.format(
                    e))
            return jpush.android(self.content, self.title)

    def _apply_custom_msg(self, msg):
        if self.custom_msg:
            msg = update_sub_dict(msg, self.custom_msg)
        return msg

    def _apply_push_type(self, msg):
        """极光 push type 的区分在推送阶段，请查看 Sender 的推送方法 `push_single_device`"""
        return msg


class JiguangSender(Sender):

    def _init_config(self):
        super(JiguangSender, self)._init_config()
        self._clent = jpush.JPush(self.access_id, self.secret_key)

    def _verify_info(self):
        super(JiguangSender, self)._verify_info()
        if self.service_provider != ServiceProvider.jiguang:
            raise ClientTypeError(
                'service provider `{}` is not `jiguang`'.format(self.service_provider))

    def check_authorization(self):
        push = self._clent.create_push()
        push.message = jpush.message('')
        push.platform = jpush.all_
        push.audience = jpush.all_
        try:
            response = push.send_validate()
        except Exception, e:
            return PushResult(PushExcetion(e))
        if response.status_code != 200:
            return PushResult(UnknowPushState(response.payload))
        else:
            return PushResult(Success(response.payload))

    def push_single_device(self, msg, target, cid_type=CidType.registration_id):
        if not isinstance(cid_type, CidType):
            raise CidTypeError('`cid_type` should be a `CidType` type value, not `{}`'
                               .format(type(cid_type)))
        push = self._clent.create_push()
        if cid_type == CidType.registration_id:
            push.audience = jpush.audience({'registration_id': [target]})
        elif cid_type == CidType.alias:
            push.audience = jpush.audience({'alias': [target]})
        else:
            raise CidTypeError('`CidType` type value `{}` unsupport.'
                               .format(cid_type.value))

        if self.client_system == ClientSystem.iOS:
            # 通知类型的使用 notification 而自定义消息使用 message
            if msg.push_type == PushType.notification:
                push.notification = jpush.notification(ios=msg.to_msg_object())
            elif msg.push_type == PushType.custom_message:
                push.message = jpush.message(msg.content, msg.title,
                                             extras=msg.custom_msg.get('extras', ''))
            else:
                raise SenderError(
                    'Unsupport push type: {}'.format(msg.push_type.name))
            push.platform = ['ios']
            push.options = {
                "apns_production": True if msg.environment == Environment.production else False}
        elif self.client_system == ClientSystem.Android:
            if msg.push_type == PushType.notification:
                push.notification = jpush.notification(android=msg.to_msg_object())
            elif msg.push_type == PushType.custom_message:
                push.message = jpush.message(msg.content, msg.title,
                                             extras=msg.custom_msg.get('extras', ''))
            else:
                raise SenderError(
                    'Unsupport push type: {}'.format(msg.push_type.name))
            push.platform = ['android']
        else:
            raise SenderError(
                'Unsupport client system type: {}.'.format(self.client_system.name))
        try:
            response = push.send()
        except Exception, e:
            return PushResult(PushExcetion(e))
        if response.status_code != 200:
            return PushResult(UnknowPushState(response.payload))
        else:
            return PushResult(Success(response.payload))



# -*- coding: utf-8 -*-
from __future__ import absolute_import

import pytest

from notification_service.push.base import (ClientSystem, Environment,
                                            PushType, ServiceProvider)
from notification_service.push.jiguang import JiguangMessage, JiguangSender
from tests.unit.push import create_Message, faker


def create_JiguangMessage(**kwargs):
    return create_Message(JiguangMessage, **kwargs)


class TestJiguangMessage(object):

    @pytest.fixture(autouse=True)
    def setup_fake_content(self):
        self.title = faker.name()
        self.content = faker.text()
        self.new_title = faker.name()
        self.new_content = faker.text()

    def test_android(self):
        msg = create_JiguangMessage(
            client_type='jiguang-android', title=self.title, content=self.content)
        assert msg.client_system == ClientSystem.Android
        assert msg.service_provider == ServiceProvider.jiguang

    def test_android_custom_msg(self):
        msg = create_JiguangMessage(client_type='jiguang-android', title=self.title,
                                    content=self.content,
                                    custom_msg={
                                        'alert': self.new_content,
                                        'title': self.new_title,
                                        'category': 'test',
                                    })
        msg_object = msg.to_msg_object()
        assert msg_object['alert'] == self.new_content
        assert msg_object['title'] == self.new_title
        assert msg_object['category'] == 'test'

    def test_android_custom_msg_with_unsupport_custom_msg(self):
        msg = create_JiguangMessage(
            client_type='jiguang-android', title=self.title, content=self.content,
            custom_msg={
                'title': self.new_title,
                'content': self.new_content,
            })
        msg_object = msg.to_msg_object()
        assert msg_object['title'] == self.title
        assert msg_object['alert'] == self.content

    def test_ios(self):
        msg = create_JiguangMessage(client_type='jiguang-ios', title=self.title,
                                    content=self.content,
                                    environment=Environment.production)
        assert msg.client_system == ClientSystem.iOS
        assert msg.service_provider == ServiceProvider.jiguang
        msg_object = msg.to_msg_object()
        assert msg_object['alert']['title'] == self.title
        assert msg_object['alert']['body'] == self.content

    def test_ios_custom_msg(self):
        msg = create_JiguangMessage(client_type='jiguang-ios', title=self.title,
                                    content=self.content,
                                    environment=Environment.development,
                                    custom_msg={
                                        'alert': {
                                            'title': self.new_title,
                                            'body': self.new_content,
                                        },
                                        'badge': 12,
                                        'sound': 'car.caf',
                                        'extras': {
                                            'key1': 'value1',
                                            'key2': 'value2',
                                        }
                                    })
        msg_object = msg.to_msg_object()
        assert msg_object['alert']['title'] == self.new_title
        assert msg_object['alert']['body'] == self.new_content
        assert msg_object['badge'] == 12
        assert msg_object['sound'] == 'car.caf'
        assert msg_object['extras'] == {
            'key1': 'value1',
            'key2': 'value2',
        }


class TestJiguangSender(object):

    @pytest.fixture()
    def push_info_ios(self):
        self.access_id = 'fe838f2bd6a1c1f92f0f3e7f'
        self.secret_key = '71fcab6158485e8b330b3178'
        self.target = '121c83f76079ae8c516'


    @pytest.fixture()
    def push_info_android(self):
        self.access_id = '069fe15945e3be255d615f92'
        self.secret_key = 'b8239f74fb1b8b2029d7fb65'
        self.target = '18071adc034c033355a'

    def test_sender_iOS(self, push_info_ios):
        client_type = 'jiguang-ios'
        msg = create_JiguangMessage(client_type=client_type,
                                    push_type=PushType.notification,
                                    environment=Environment.development,
                                    custom_msg={
                                        'alert': {
                                            'title': 'title of jiguang',
                                            'body': 'content of jiguang',
                                        },
                                        'sound': 'car.caf',
                                        'extras': {
                                            'product_key': 'asdfasfdasd',
                                            'did': 'value2',
                                        }
                                    })
        sender = JiguangSender(
            access_id=self.access_id, secret_key=self.secret_key,
            client_type=client_type)
        assert sender.push_single_device(msg, self.target).is_success

    def test_sender_Android(self, push_info_android):
        client_type = 'jiguang-android'
        msg = create_JiguangMessage(client_type=client_type,
                                    push_type=PushType.notification,
                                    custom_msg={
                                        'alert': 'content of jiguang',
                                        'title': 'title of jiguang',
                                        'category': 'test',
                                    })
        sender = JiguangSender(
            access_id=self.access_id, secret_key=self.secret_key,
            client_type=client_type)
        assert sender.push_single_device(msg, self.target).is_success

    def test_check_authorization(self, push_info_ios):
        for clien_type in ['jiguang-ios', 'jiguang-android']:
            sender = JiguangSender(
                access_id=self.access_id, secret_key=self.secret_key,
                client_type=clien_type)
            assert sender.check_authorization().is_success



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


class PushError(Exception):
    pass


class ClientTypeError(PushError, ValueError):
    """Client Type infomation error"""


class CidTypeError(PushError, ValueError):
    """cid type error"""


class SenderError(PushError):
    pass


class AWSSenderError(SenderError):
    pass


class RequireFieldsError(Exception):

    need_fields = set()

    def __init__(self, fields):
        self.need_fields = fields

    def __str__(self):
        return 'Fields {} is need.'.format(tuple(self.need_fields))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from unittest import TestCase

import mock
from faker import Faker

from notification_service.push.base import ClientSystem, ServiceProvider, Environment
from notification_service.push.xinge import XingeMessage, XingeSender
from tests.unit.push import TestMessage, create_Message

faker = Faker()


def create_XingeMessage(**kwargs):
    return create_Message(XingeMessage, **kwargs)


class test_XingeMessage(TestMessage):

    def test_android(self):
        msg = create_XingeMessage(
            client_type='xinge-android', title=self.title, content=self.content)
        assert msg.client_system == ClientSystem.Android
        assert msg.service_provider == ServiceProvider.xinge

    def test_android_custom_msg(self):
        msg = create_XingeMessage(
            client_type='xinge-android', title=self.title, content=self.content,
            custom_msg={
                'title': self.new_title,
                'content': self.new_content,
            })
        msg_object = msg.to_msg_object()
        assert msg_object.raw['title'] == self.new_title
        assert msg_object.raw['content'] == self.new_content

    def test_ios(self):
        msg = create_XingeMessage(client_type='xinge-ios', title=self.title,
                                  content=self.content, environment=Environment.production)
        assert msg.client_system == ClientSystem.iOS
        assert msg.service_provider == ServiceProvider.xinge

    def test_ios_custom_msg(self):
        environment = Environment.production
        msg = create_XingeMessage(client_type='xinge-ios', title=self.title,
                                  content=self.content, environment=environment,
                                  custom_msg={
                                      'aps': {
                                          'alert': {
                                              'title': self.new_title,
                                              'body': self.new_content,
                                          }
                                      }
                                  })
        msg_object = msg.to_msg_object()
        assert msg_object.raw['aps']['alert']['title'] == self.new_title
        assert msg_object.raw['aps']['alert']['body'] == self.new_content


class test_XingeSender(TestCase):

    def setUp(self):
        self.access_id = '2200321268'
        self.secret_key = '2a020ed64ef2e31e352f076d7460543a'
        self.target = 'bc648aa880d4757127a4433ddda073b19f8454a60bd53b6da952890ca11848cf'
        self.client_types = ['xinge-ios', 'xinge-android']

        # self.mock_PushSingleDevice_success = mock.Mock(return_value=(0, ''))
        self.mock_QueryDeviceCount_success = mock.Mock(return_value=(0, 12))

    def test_sender(self):
        for client_type in self.client_types:
            msg = create_XingeMessage(client_type=client_type)
            sender = XingeSender(
                access_id=self.access_id, secret_key=self.secret_key,
                client_type=client_type, environment=Environment.development)
            # sender._client.PushSingleDevice = self.mock_PushSingleDevice_success
            assert sender.push_single_device(msg, self.target).is_success

    def test_check_authorization(self):
        for clien_type in self.client_types:
            sender = XingeSender(
                access_id=self.access_id, secret_key=self.secret_key,
                client_type=clien_type)
            assert sender.check_authorization().is_success



#!/usr/bin/env python
from notification_service.jpush import common
import json

class Device(object):
    """Device info query/update..

    """
    def __init__(self, jpush):
        self._jpush = jpush
        self.entity = None

    def send(self, method, url, body, content_type=None, version=3):
        """Send the request

        """
        response = self._jpush._request(method, body,
            url, content_type, version=3)
        return DeviceResponse(response)

    def send2(self, method, url, body, content_type=None, version=3):
        response = self._jpush._request(method, body,
            url, content_type, version=3)
        return response.json()

    def get_taglist(self):
        """Get deviceinfo with registration id.
        """
        url = common.TAGLIST_URL
        body = None
        info = self.send2("GET", url, body)
        err  = info.get('error')
        if err:
            return {'errmsg': err.get('message')}
        return info

    def get_deviceinfo(self, registration_id):
        """Get deviceinfo with registration id.
        """
        url = common.DEVICE_URL + registration_id + "/"
        body = None
        info = self.send("GET", url, body)
        print (info)
        return info

    def set_deviceinfo(self, registration_id, entity):
        """Update deviceinfo with registration id.
        """
        url = common.DEVICE_URL + registration_id + "/"
        body = json.dumps(entity)
        info = self.send("POST", url, body)
        print (url), (body), (info)
        return info

    def delete_tag(self, tag, platform=None):
        """Delete registration id tag.
        """
        url = common.TAG_URL + tag + "/"
        body = None
        if platform:
            body = platform
        print (url), (body)
        info = self.send("DELETE", url, body)
        print (info)
        return info

    def update_tagusers(self, tag, entity):
        """Add/Remove specified tag users.
        """
        url = common.TAG_URL + tag + "/"
        body = json.dumps(entity)
        print (url), (body)
        info = self.send("POST", url, body)
        print (info)
        return info

    def check_taguserexist(self, tag, registration_id):
        """Check registration id whether in tag.
        """
        url = common.TAG_URL + tag + "/registration_ids/" + registration_id
        body = registration_id
        print (url), (registration_id)
        info = self.send("GET", url, body)
        print (info)
        return info

    def delete_alias(self, alias, platform=None):
        """Delete appkey alias.
        """
        url = common.ALIAS_URL + alias + "/"
        body = None
        if platform:
            body = platform
        print (url), (body)
        info = self.send("DELETE", url, body)
        print (info)
        return info

    def get_aliasuser(self, alias, platform=None):
        """Get appkey alias users.
        """
        url = common.ALIAS_URL + alias + "/"
        body = None
        if platform:
            body = platform
        print (url), (body)
        info = self.send("GET", url, body)
        print (info)
        return info

class DeviceResponse(object):
    """Response to a successful device request send.

    Right now this is a fairly simple wrapper around the json payload response,
    but making it an object gives us some flexibility to add functionality
    later.

    """
    payload = None

    def __init__(self, response):
        if 0 != len(response.content):
            data = response.json()
            self.payload = data
        elif 200 == response.status_code:
            self.payload = "success"

    def __str__(self):
        return str(self.payload)



# coding=utf-8
from  notification_service import jpush

class Jiguang_Push(object):

    def __init__(self, app_key, master_secret):
        self.app_key = app_key
        self.master_secret = master_secret

    def set_alias(self, cid):
        # 设置jpush设备别名
        _jpush = jpush.JPush(self.app_key, self.master_secret)
        device = _jpush.create_device()
        alias = cid
        platform = "android,ios"
        device.get_aliasuser(alias, platform)

    def push_client_message(self, cid, content, title="通知"):
        _jpush = jpush.JPush(self.app_key, self.master_secret)
        push = _jpush.create_push()
        push.audience = jpush.audience(
            jpush.alias(cid)
        )
        push.message = jpush.message(content, title)
        push.platform = jpush.all_
        try:
            ret = push.send()
        except Exception, e:
            return {"error_msg": str(e)}
        return ret

    def push_single_device(self, cid, content, title="通知", type=1, client_type="jiguang-android", deploy_status=True, cid_type=1, extras_msg={}, jiguang_custom={}):
        _jpush = jpush.JPush(self.app_key, self.master_secret)
        push = _jpush.create_push()
        if cid_type == 1:
            push.audience = jpush.audience(
                jpush.alias(cid)
            )
        elif cid_type == 2:
            push.audience = jpush.audience(
                jpush.registration_id(cid)
            )
        android_msg = None
        ios_msg = None
        if client_type == "jiguang-ios":
            msg = {'title': title}
            if extras_msg:
                msg.update(extras_msg)
            ios_msg = jpush.ios(alert=content, badge="+1", sound="happy.caf", extras=msg)
            ios_custom = jiguang_custom.get('ios', {})
            if ios_custom:
                badge = ios_custom.get('badge')
                if badge is not None:
                    ios_msg['badge'] = badge
                sound = ios_custom.get('sound')
                if sound is not None:
                    ios_msg['sound'] = sound
                content_available = ios_custom.get('content-available')
                if content_available is not None:
                    ios_msg['content-available'] = content_available
                mutable_available = ios_custom.get('mutable-available')
                if mutable_available is not None:
                    ios_msg['mutable-available'] = mutable_available
                extras = ios_custom.get('extras', {})
                if extras:
                    ios_msg['extras'].update(extras)
                category = ios_custom.get('category')
                if category is not None:
                    ios_msg['category'] = category
        elif client_type == 'jiguang-android':
            msg = {'title': title}
            if extras_msg:
                msg.update(extras_msg)
            android_msg = jpush.android(alert=content, title=title, extras=msg)
            android_custom = jiguang_custom.get('android', {})
            if android_custom:
                builder_id = android_custom.get('builder_id')
                if builder_id is not None:
                    android_msg['builder_id'] = builder_id
                priority = android_custom.get('priority')
                if priority is not None:
                    android_msg['priority'] = priority
                category = android_custom.get('category')
                if category is not None:
                    android_msg['category'] = category
                alert_type = android_custom.get('alert_type')
                if alert_type is not None:
                    android_msg['alert_type'] = alert_type
                big_text = android_custom.get('big_text')
                if big_text is not None:
                    android_msg['big_text'] = big_text
                extras = android_custom.get('extras', {})
                if extras:
                    android_msg['extras'].update(extras)
                big_pic_path = android_custom.get('big_pic_path')
                if big_pic_path is not None:
                    android_msg['big_pic_path'] = big_pic_path
        if type == 1:
            push.notification = jpush.notification(alert=title, android=android_msg, ios=ios_msg)
        elif type == 2:
            custom_extras = {}
            if android_msg:
                android_extras = android_msg.get('extras')
                if android_extras:
                    custom_extras.update(android_extras)
            if ios_msg:
                ios_extras = ios_msg.get('extras')
                if ios_extras:
                    custom_extras.update(ios_extras)
            push.message = jpush.message(content, title, extras=custom_extras)
        push.platform = jpush.all_
        push.options = {"apns_production": deploy_status}
        try:
            ret = push.send()
        except Exception, e:
            return {"error_msg": str(e)}
        return ret



import re

# Valid autobadge values: auto, +N, -N
VALID_AUTOBADGE = re.compile(r'^(auto|[+-][\d]+)$')

def notification(alert=None, ios=None, android=None, winphone=None):
    """Create a notification payload.

    :keyword alert: A simple text alert, applicable for all platforms.
    :keyword ios: An iOS platform override, as generated by :py:func:`ios`.
    :keyword android: An Android platform override, as generated by :py:func:`android`.
    :keyword winphone: A MPNS platform override, as generated by :py:func:`mpns`.

    """
    payload = {}
    if alert is not None:
        payload['alert'] = alert
    if ios is not None:
        payload['ios'] = ios
    if android is not None:
        payload['android'] = android
    if winphone is not None:
        payload['winphone'] = winphone
    if not payload:
        raise ValueError("Notification body may not be empty")
    return payload


def ios(alert=None, badge=None, sound=None, content_available=False,
        extras=None, sound_disable=False):
    """iOS/APNS specific platform override payload.

    :keyword alert: iOS format alert, as either a string or dictionary.
    :keyword badge: An integer badge value or an *autobadge* string.
    :keyword sound: An string sound file to play.
    :keyword content_available: If True, pass on the content-available command
        for Newsstand iOS applications.
    :keyword extra: A set of key/value pairs to include in the push payload
        sent to the device.
    :keyword sound_disalbe: Disable sound to implement slient push.

    >>> ios(alert='Hello!', sound='cat.caf',
    ...     extra={'articleid': '12345'})
    {'sound': 'cat.caf', 'extra': {'articleid': '12345'}, 'alert': 'Hello!'}

    """
    payload = {}
    if alert is not None:
        if not isinstance(alert, str) or isinstance(alert, dict):
            raise ValueError("iOS alert must be a string or dictionary")
        payload['alert'] = alert
    if badge is not None:
        if not (isinstance(badge, str) or isinstance(badge, int)):
            raise ValueError("iOS badge must be an integer or string")
        if isinstance(badge, str) and not VALID_AUTOBADGE.match(badge):
            raise ValueError("Invalid iOS autobadge value")
        payload['badge'] = badge
    if not sound_disable:
        if sound is not None:
            payload['sound'] = sound
        else:
            payload['sound'] = 'default'
    if content_available:
        payload['content-available'] = 1
    if extras is not None:
        payload['extras'] = extras
    return payload

def android(alert, title=None, builder_id=None, extras=None):
    """Android specific platform override payload.

    All keyword arguments are optional.

    :keyword alert: String alert text.
    :keyword title: String
    :keyword builder_id: Integer
    :keyword extras: A set of key/value pairs to include in the push payload
        sent to the device.
    """
    payload = {}
    if alert is not None:
        payload['alert'] = alert
    if title is not None:
        payload['title'] = title
    if builder_id is not None:
        payload['builder_id'] = builder_id
    if extras is not None:
        payload['extras'] = extras
    return payload


def winphone(alert, title=None, _open_page=None, extras=None):
    """MPNS specific platform override payload.

    Must include exactly one of ``alert``, ``title``, ``_open_page``, or ``extras``.

    """
    if len(list(filter(None, (alert, _open_page, title)))) != 1:
        raise ValueError("MPNS payload must have one notification type.")
    payload = {}
    if alert is not None:
        payload['alert'] = alert
    if title is not None:
        payload['title'] = title
    if _open_page is not None:
        payload['_open_page'] = _open_page
    if extras is not None:
        payload['extras'] = extras
    return payload

def message(msg_content, title=None, content_type=None, extras=None):
    """Inner-conn push message payload creation.

    :param msg_content: Required, string
    :param title: Optional, string
    :keyword content_type: Optional, MIME type of the body
    :keyword extras: Optional, dictionary of string values.

    """
    payload = {
        'msg_content': msg_content,
    }
    if title is not None:
        payload['title'] = title
    if content_type is not None:
        payload['content_type'] = content_type
    if extras is not None:
        payload['extras'] = extras
    return payload

def platform(*types):
    """Create a platform specifier.

    >>> platform('ios', 'winphone')
    ['ios', 'winphone']
    >>> platform('ios', 'symbian')
    Traceback (most recent call last):
        ...
    ValueError: Invalid platform 'symbian'

    """
    if len(types) == 1 and types[0] == 'all':
        return 'all'
    for t in types:
        if t not in ('ios', 'android', 'winphone'):
            raise ValueError("Invalid platform '%s'" % t)
    return [t for t in types]

def options(options):
    """Create options object."""
    return {"options": options}

def audience(*types):
    """Select audience that match all of the given selectors.

    >>> audience(tag('sports'), tag_and('business'))
    {'audience': {'tag':['sports'], 'tag_and':['business']}}

    """
    if 1 == len(types) and 'all' == types[0]:
        return "all"
    audience = {}
    for t in types:
        for key in t:
            if key not in ('tag', 'tag_and', 'alias', 'registration_id'):
                raise ValueError("Invalid audience '%s'" % t)
            audience[key] = t[key]
    return audience



#!/usr/bin/python
# coding=utf-8

import time
import urllib
import hashlib
import json
import requests


###
# 百度云推送服务端SDK Python版本
# @version 1.0.0
###


class Channel(object):
    """
    Channel类提供百度云推送服务端SDK的Python版本，
    用户首先实例化这个类，设置自己的apiKey和secretKey，即可使用Push服务接口
    """
    # 用户发起请求时的unix时间戳。本次请求签名的有效时间为该时间戳+10分钟
    TIMESTAMP = 'timestamp'

    # 用户指定本次请求签名的失效时间。格式为unix时间戳形式
    EXPIRES = 'expires'

    # API版本号，默认使用最高版本
    VERSION = 'v'

    # 通道标识
    CHANNEL_ID = 'channel_id'

    # 用户标识
    USER_ID = 'user_id'

    # 设备类型。如果存在此字段，则只返回该设备类型的绑定关系；默认不区分设备类型
    DEVICE_TYPE = 'device_type'

    # 可选设备类型
    DEVICE_BROWSER = 1
    DEVICE_PC = 2
    DEVICE_ANDRIOD = 3
    DEVICE_IOS = 4
    DEVICE_WINDOWSPHONE = 5

    # 查询起始页码，默认为0
    START = 'start'

    # 一次查询条数，默认为10
    LIMIT = 'limit'

    # 指定消息内容，单个消息为单独字符串
    MESSAGES = 'messages'

    # 删除的消息id列表，由一个或多个msg_id组成，多个用json数组表示
    MSG_IDS = 'msg_ids'

    # 消息标识。指定消息标识，自动覆盖相同消息标识的消息，只支持android、browser、pc三种设备类型。
    MSG_KEYS = 'msg_keys'

    # 消息类型，0：消息（透传），1：通知，默认为0
    MESSAGE_TYPE = 'message_type'

    # 可选消息类型
    PUSH_MESSAGE = 0
    PUSH_NOTIFICATION = 1

    # 指定消息的过期时间，默认为86400秒。
    MESSAGE_EXPIRES = 'message_expires'

    # 标签名，可按标签分组
    TAG_NAME = 'tag'

    # 标签信息
    TAG_INFO = 'info'

    # 标签ID
    TAG_ID = 'tid'

    # 访问令牌，明文AK，可从此值获得App的信息，配合sign中的sk做合法性身份认证。
    API_KEY = 'apikey'

    # 开发者密钥，用于调用HTTP API时进行签名加密，以证明开发者的合法身份。
    SECRET_KEY = 'secret_key'

    # Channel常量
    SIGN = 'sign'
    METHOD = 'method'
    HOST = 'host'
    PRODUCT = 'channel'
    DEFAULT_HOST = 'channel.api.duapp.com'

    # 证书相关常量
    NAME = 'name'
    DESCRIPTION = 'description'
    CERT = 'cert'
    RELEASE_CERT = 'release_cert'
    DEV_CERT = 'dev_cert'

    # 推送类型
    PUSH_TYPE = 'push_type'

    # 可选推送类型
    PUSH_TO_USER = 1
    PUSH_TO_TAG = 2
    PUSH_TO_ALL = 3

    # Channel错误常量
    CHANNEL_SDK_SYS = 1
    CHANNEL_SDK_INIT_FAIL = 2
    CHANNEL_SDK_PARAM = 3
    CHANNEL_SDK_HTTP_STATUS_ERROR_AND_RESULT_ERROR = 4
    CHANNEL_SDK_HTTP_STATUS_OK_BUT_RESULT_ERROR = 5

    ###
    # 对外接口
    ###

    def __init__(self, apiKey, secretKey):
        self._apiKey = apiKey
        self._secretKey = secretKey
        self._requestId = 0
        self._method_channel_in_body = ['push_msg', 'set_tag', 'fetch_tag', 'delete_tag', 'query_user_tags']

    def setApiKey(self, apiKey):
        self._apiKey = apiKey

    def setSecretKey(self, secretKey):
        self._secretKey = secretKey

    def getRequestId(self):
        return self._requestId

    # 查询设备、应用、用户与百度Channel的绑定关系
    def queryBindList(self, userId, optional=None):
        """
        参数：
            str userId：用户ID号
            dict optional：可选参数
        返回值：
            成功：python字典；失败：False
        """
        tmpArgs = [userId, optional]
        arrArgs = self._mergeArgs([Channel.USER_ID], tmpArgs)
        arrArgs[Channel.METHOD] = 'query_bindlist'
        return self._commonProcess(arrArgs)

    # 推送消息，该接口可用于推送单个人、一群人、所有人以及固定设备的使用场景
    def pushMessage(self, push_type, messages, message_keys, optional=None):
        """
        参数：
            push_type：推送消息的类型
            messages：消息内容
            message_keys：消息key
            optional：可选参数
        返回值：
            成功：python字典；失败：False
        """
        tmpArgs = [push_type, messages, message_keys, optional]
        arrArgs = self._mergeArgs([Channel.PUSH_TYPE, Channel.MESSAGES, Channel.MSG_KEYS], tmpArgs)
        arrArgs[Channel.METHOD] = 'push_msg'
        arrArgs[Channel.PUSH_TYPE] = push_type
        arrArgs[Channel.MESSAGES] = json.dumps(arrArgs[Channel.MESSAGES])
        arrArgs[Channel.MSG_KEYS] = json.dumps(arrArgs[Channel.MSG_KEYS])
        return self._commonProcess(arrArgs)

    def pushSingle(self, data=None):
        """
        返回值：
            成功：python字典；失败：False
        """
        url = 'http://api.tuisong.baidu.com/rest/3.0/push/single_device'
        return self._request_result(data, url)

    def _request_result(self, data, url):
        self._adjustOpt2(data)
        ret = self._baseControl2(data, url)
        result = ret.json()
        self._requestId = result['request_id']
        if ret.status_code == requests.codes.ok:
            return result
        return result

    def _adjustOpt2(self, opt):
        if Channel.TIMESTAMP not in opt:
            opt[Channel.TIMESTAMP] = int(time.time())
        opt[Channel.API_KEY] = self._apiKey
        if Channel.SECRET_KEY in opt:
            del opt[Channel.SECRET_KEY]

    def _baseControl2(self, data, url):
        http_method = 'POST'
        data[Channel.SIGN] = self._genSign(http_method, url, data)
        headers = dict()
        headers['Content-Type'] = 'application/x-www-form-urlencoded;charset=UTF-8'
        headers['User-Agent'] = 'BCCS_SDK/3.0' +\
                                'system_info' +\
                                'python/2.7.3 (Baidu Push Server SDK V3.0.0) cli/Unknown'
        print data
        return requests.post(url, data=data, headers=headers)

    def _baseControl(self, opt):
        resource = 'channel'
        if Channel.CHANNEL_ID in opt:
            if opt[Channel.CHANNEL_ID] and opt[Channel.METHOD] not in self._method_channel_in_body:
                resource = opt[Channel.CHANNEL_ID]
                del opt[Channel.CHANNEL_ID]

        host = opt[Channel.HOST]
        del opt[Channel.HOST]

        url = 'http://' + host + '/rest/3.0/' + Channel.PRODUCT + '/'
        url += resource
        http_method = 'POST'
        opt[Channel.SIGN] = self._genSign(http_method, url, opt)

        headers = dict()
        headers['Content-Type'] = 'application/x-www-form-urlencoded'
        headers['User-Agent'] = 'Baidu Channel Service Pythonsdk Client'
        return requests.post(url, data=opt, headers=headers)


    # 判断设备、应用、用户与Channel的绑定关系是否存在
    def verifyBind(self, userId, optional=None):
        """
        参数：
            userId：用户id
            optional：可选参数
        返回值：
            成功：python数组；失败：False
        """
        tmpArgs = [userId, optional]
        arrArgs = self._mergeArgs([Channel.USER_ID], tmpArgs)
        arrArgs[Channel.METHOD] = 'verify_bind'
        return self._commonProcess(arrArgs)

    # 查询离线消息
    def fetchMessage(self, userId, optional=None):
        """
        参数：
            userId：用户id
            optional：可选参数
        返回值：
            成功：python字典；失败：False
        """
        tmpArgs = [userId, optional]
        arrArgs = self._mergeArgs([Channel.USER_ID], tmpArgs)
        arrArgs[Channel.METHOD] = 'fetch_msg'
        return self._commonProcess(arrArgs)

    # 查询离线消息的个数
    def fetchMessageCount(self, userId, optional=None):
        """
        参数：
            userId：用户id
            optional：可选参数
        返回值：
            成功：python字典；失败：False
        """
        tmpArgs = [userId, optional]
        arrArgs = self._mergeArgs([Channel.USER_ID], tmpArgs)
        arrArgs[Channel.METHOD] = 'fetch_msgcount'
        return self._commonProcess(arrArgs)

    # 删除离线消息
    def deleteMessage(self, userId, msgId, optional=None):
        """
        参数：
            userId：用户id
            msgIds：消息id
            optional：可选参数
        返回值：
            成功：python字典；失败：False
        """
        tmpArgs = [userId, msgId, optional]
        arrArgs = self._mergeArgs([Channel.USER_ID, Channel.MSG_IDS], tmpArgs)
        arrArgs[Channel.METHOD] = 'delete_msg'
        if isinstance(arrArgs[Channel.MSG_IDS], list):
            arrArgs[Channel.MSG_IDS] = json.dumps(arrArgs[Channel.MSG_IDS])
        return self._commonProcess(arrArgs)

    # 服务器端设置用户标签。
    # 当该标签不存在时，服务端将会创建该标签。特别地，当user_id被提交时，服务端将会完成用户和tag的绑定操作。
    def setTag(self, tagName, optional=None):
        """
        参数：
            tagName：标签
            optional：可选参数
        返回值：
            成功：python字典；失败：False
        """
        tmpArgs = [tagName, optional]
        arrArgs = self._mergeArgs([Channel.TAG_NAME], tmpArgs)
        arrArgs[Channel.METHOD] = 'set_tag'
        return self._commonProcess(arrArgs)

    # App Server查询应用标签
    def fetchTag(self, optional=None):
        """
        参数：
            optional：可选参数
        返回值：
            成功：python字典；失败：False
        """
        tmpArgs = [optional]
        arrArgs = self._mergeArgs([], tmpArgs)
        arrArgs[Channel.METHOD] = 'fetch_tag'
        return self._commonProcess(arrArgs)

    # 服务端删除用户标签。
    # 特别地，当user_id被提交时，服务端将只会完成解除该用户与tag绑定关系的操作。
    # 注意：该操作不可恢复，请谨慎使用。
    def deleteTag(self, tagName, optional=None):
        """
        参数：
            tagName：标签
            optional：可选参数
        返回值：
            成功：python字典；失败：False
        """
        tmpArgs = [tagName, optional]
        arrArgs = self._mergeArgs([Channel.TAG_NAME], tmpArgs)
        arrArgs[Channel.METHOD] = 'delete_tag'
        return self._commonProcess(arrArgs)

    # App Server查询用户所属的标签列表
    def queryUserTag(self, userId, optional=None):
        """
        参数：
            userId：用户id
            optional：可选参数
        返回值：
            成功：python字典；失败：False
        """
        tmpArgs = [userId, optional]
        arrArgs = self._mergeArgs([Channel.USER_ID], tmpArgs)
        arrArgs[Channel.METHOD] = 'query_user_tags'
        return self._commonProcess(arrArgs)

    # 根据channel_id查询设备类型
    def queryDeviceType(self, channelId, optional=None):
        """
        根据channelId查询设备类型
        参数：
            ChannelId：用户Channel的ID号
            optional：可选参数
        返回值：
            成功：python字典；失败：False
        """
        tmpArgs = [channelId, optional]
        arrArgs = self._mergeArgs([Channel.CHANNEL_ID], tmpArgs)
        arrArgs[Channel.METHOD] = 'query_device_type'
        return self._commonProcess(arrArgs)


    ###
    # 内部函数
    ###

    def _checkString(self, string, minLen, maxLen):
        return minLen <= len(string) <= maxLen

    def _adjustOpt(self, opt):
        if Channel.TIMESTAMP not in opt:
            opt[Channel.TIMESTAMP] = int(time.time())
        opt[Channel.HOST] = Channel.DEFAULT_HOST
        opt[Channel.API_KEY] = self._apiKey
        if Channel.SECRET_KEY in opt:
            del opt[Channel.SECRET_KEY]

    def _genSign(self, method, url, arrContent):
        gather = method + url
        keys = arrContent.keys()
        keys.sort()
        for key in keys:
            gather += key + '=' + str(arrContent[key])
        gather += self._secretKey
        sign = hashlib.md5(urllib.quote_plus(gather))
        return sign.hexdigest()

    def _baseControl(self, opt):
        resource = 'channel'
        if Channel.CHANNEL_ID in opt:
            if opt[Channel.CHANNEL_ID] and opt[Channel.METHOD] not in self._method_channel_in_body:
                resource = opt[Channel.CHANNEL_ID]
                del opt[Channel.CHANNEL_ID]

        host = opt[Channel.HOST]
        del opt[Channel.HOST]

        url = 'http://' + host + '/rest/2.0/' + Channel.PRODUCT + '/'
        url += resource
        http_method = 'POST'
        opt[Channel.SIGN] = self._genSign(http_method, url, opt)

        headers = dict()
        headers['Content-Type'] = 'application/x-www-form-urlencoded'
        headers['User-Agent'] = 'Baidu Channel Service Pythonsdk Client'

        return requests.post(url, data=opt, headers=headers)

    def _commonProcess(self, paramOpt):
        self._adjustOpt(paramOpt)
        ret = self._baseControl(paramOpt)
        result = ret.json()
        self._requestId = result['request_id']

        if ret.status_code == requests.codes.ok:
            return result
        return result

    def _mergeArgs(self, arrNeed, tmpArgs):
        arrArgs = dict()

        if not arrNeed and not tmpArgs:
            return arrArgs

        if len(tmpArgs)-1 != len(arrNeed) and len(tmpArgs) != len(arrNeed):
            keys = '('
            for key in arrNeed:
                keys += key + ','
            if key[-1] == '' and key[-2] == ',':
                keys = keys[0:-2]
            keys += ')'
            raise Exception('invalid sdk, params, params' + keys + 'are need', Channel.CHANNEL_SDK_PARAM)

        if len(tmpArgs)-1 == len(arrNeed) and tmpArgs[-1] is not None and (not isinstance(tmpArgs[-1], dict)):
            raise Exception('invalid sdk params, optional param must bean dict', Channel.CHANNEL_SDK_PARAM)

        idx = 0
        if isinstance(arrNeed, list):
            for key in arrNeed:
                if tmpArgs[idx] is None:
                    raise Exception('lack param ' + key, Channel.CHANNEL_SDK_PARAM)
                arrArgs[key] = tmpArgs[idx]
                idx = idx + 1

        if len(tmpArgs) == idx + 1 and tmpArgs[idx] is not None:
            for (key, value) in tmpArgs[idx].items():
                if key not in arrArgs and value is not None:
                    arrArgs[key] = value

        return arrArgs



SUCCESS = (0, '')

ACCESS_ID_NOT_NUMBER = (-800, 'accessId is not number.')
ACCESS_ID_ILLEGAL = (-801, 'accessId is illegal.')

SECRET_KEY_TYPE_ERROR = (-810, 'secretKey is not unicode type.')

TOKEN_TYPE_ERROR = (-810, 'token is not unicode type.')

CLIENT_TYPE_ILLEGAL = (-820, 'client_type is illegal.')

CONTENT_TYPE_ERROR = (-830, 'content is not unicode type.')

TITLE_TYPE_ERROR = (-840, 'title is not unicode type.')

MSG_TYPE_ILLEGAL = (-850, 'msg_type is illegal.')

CUSTOM_INFO_TYPE_ERROR = (-860, 'custom_info is not dict type.')

ENVIRONMENT_ILLEGAL = (-870, 'environment is illegal.')


"""Python package for using the JPush API"""
from .core import JPush
from .common import JPushFailure, Unauthorized

__version__ = '3.1.2'
VERSION = tuple(map(int,  __version__.split('.')))

from .push import (
    Push,
    all_,
    tag,
    tag_and,
    alias,
    registration_id,
    notification,
    ios,
    android,
    winphone,
    platform,
    audience,
    options,
    message,
)

from .device import (
    Device,
    add,
    remove,
    device_tag,
    device_alias,
    device_regid,
)

__all__ = [
    JPush,
    JPushFailure,
    Unauthorized,
    all_,
    Push,
    tag,
    tag_and,
    alias,
    registration_id,
    notification,
    ios,
    android,
    winphone,
    message,
    platform,
    audience,
    options,
    Device,
    add,
    remove,
    device_tag,
    device_alias,
    device_regid,
]

# Silence urllib3 INFO logging by default

import logging
logging.getLogger('requests.packages.urllib3.connectionpool').setLevel(logging.WARNING)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import


from .states import Success


class ResultBase(object):
    """Base class for results."""
    state = None
    data = None

    def __init__(self, state=Success(), data=None):
        self.state = state
        if data:
            self.data = data

    @property
    def is_success(self):
        return self.state.is_success

    def __unicode__(self):
        return self.state.__unicode__()

    def __str__(self):
        return self.state.__str__()


class PushResult(ResultBase):
    """App push state."""



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

import json

import boto3
from enum import Enum

from notification_service.exceptions import (ClientTypeError, AWSSenderError)
from notification_service.result import PushResult
from notification_service.states import (
    UnknowPushState, PushExcetion, CheckAuthorizationExcetion, Success)
from notification_service.utils import from_ARN_to_region_name, update_sub_dict
from .base import (Sender, Message, ClientSystem,
                   ServiceProvider, PushType)


class AWSiOSContentKey(Enum):
    development = 'APNS_SANDBOX'
    production = 'APNS'


class AWSMessage(Message):
    _iOS_content_key = None

    def to_msg_object(self):
        if self.client_system is ClientSystem.iOS:
            return self._to_msg_object_iOS()
        if self.client_system is ClientSystem.Android:
            return self._to_msg_object_GCM()

    def _to_msg_object_GCM(self):
        data = {
            'GCM': {
                'notification': {
                    'title': self.title,
                    'body': self.content,
                    'sound': 'default',
                },
                'data': {
                    'system': 'android',
                }
            }
        }
        return self._encode_msg_content(
            self._apply_push_type(self._apply_custom_msg(data)),
            'GCM'
        )

    def _to_msg_object_iOS(self):
        data = {
            self._iOS_content_key: {
                'aps': {
                    'alert': {
                        'title': self.title,
                        'body': self.content,
                    },
                    'sound': 'default',
                }
            }
        }
        return self._encode_msg_content(
            self._apply_push_type(self._apply_custom_msg(data)),
            self._iOS_content_key
        )

    def _apply_custom_msg(self, data):
        if self.custom_msg:
            return update_sub_dict(data, self.custom_msg)
        return data

    def _apply_push_type(self, data):
        if self.push_type is PushType.custom_message:
            if self.client_system is ClientSystem.iOS:
                return self._apply_push_type_iOS(data)
            if self.client_system is ClientSystem.Android:
                return self._apply_push_type_GCM(data)
        return data

    def _apply_push_type_iOS(self, data):
        if self.push_type is PushType.custom_message:
            data[self._iOS_content_key]['aps']['content-available'] = 1
            data[self._iOS_content_key]['aps'].pop('alert', None)
            data[self._iOS_content_key]['aps'].pop('sound', None)
        return data

    def _apply_push_type_GCM(self, data):
        if self.push_type is PushType.custom_message:
            data['GCM'].pop('notification', None)
        return data

    def _encode_msg_content(self, data, key):
        data[key] = json.dumps(data[key])
        return data

    @property
    def environment(self):
        return self._environment

    @environment.setter
    def environment(self, value):
        self._environment = value
        self._iOS_content_key = AWSiOSContentKey.__members__[
            self.environment.value].value


class AWSSender(Sender):
    region_name = ''
    message_structure = 'json'
    _client = None
    application_arn = ''

    def __init__(self, *args, **kwargs):
        """init

        Arguments:
            access_id {str} -- Access key ID
            secret_key {str} -- Secret access key
            application_arn {str} -- Application ARN
            region_name {str} -- Region name of ARN
        """
        super(AWSSender, self).__init__(*args, **kwargs)

    def _init_config(self):
        super(AWSSender, self)._init_config()
        if self.application_arn:
            self.region_name = from_ARN_to_region_name(self.application_arn)
        self._client = boto3.client('sns',
                                    aws_access_key_id=self.access_id,
                                    aws_secret_access_key=self.secret_key,
                                    region_name=self.region_name)

    def _verify_info(self):
        super(AWSSender, self)._verify_info()
        if self.service_provider != ServiceProvider.AWS:
            raise ClientTypeError(
                'service provider `{}` is not `AWS`'.format(self.service_provider))
        if self.region_name == '':
            raise AWSSenderError('`region_name` should not be empty.')

    def check_authorization(self):
        try:
            response = self._client.get_platform_application_attributes(
                PlatformApplicationArn=self.application_arn)
        except Exception, e:
            return PushResult(CheckAuthorizationExcetion(e))
        if response['ResponseMetadata']['HTTPStatusCode'] != 200:
            return PushResult(UnknowPushState(response))
        else:
            return PushResult(Success(response))

    def create_platform_endpoint(self, token):
        try:
            response = self._client.create_platform_endpoint(
                PlatformApplicationArn=self.application_arn, Token=token)
        except Exception, e:
            return PushResult(PushExcetion(e))
        if response['ResponseMetadata']['HTTPStatusCode'] != 200:
            return PushResult(UnknowPushState(response))
        else:
            return PushResult(Success(response), data=response['EndpointArn'])

    def push_single_device(self, msg, target):
        try:
            response = self._client.publish(
                TargetArn=target, MessageStructure='json',
                Message=json.dumps(msg.to_msg_object()))
        except Exception, e:
            return PushResult(PushExcetion(e))
        if response['ResponseMetadata']['HTTPStatusCode'] != 200:
            return PushResult(UnknowPushState(response))
        else:
            return PushResult(Success(response))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from unittest import TestCase
from operator import methodcaller

from faker import Faker

from notification_service.push.base import Environment

faker = Faker()


def create_Message(cls, **kwargs):
    need_faker_fields = [('title', 'name'), ('content', 'text')]
    for field_name, method_name in need_faker_fields:
        if not field_name in kwargs:
            kwargs[field_name] = methodcaller(method_name)(faker)
    if 'ios' in kwargs['client_type'] and not 'environment' in kwargs:
        kwargs.update(environment=Environment.production)
    return cls(**kwargs)


class TestMessage(TestCase):

    def setUp(self):
        self.title = faker.name()
        self.content = faker.text()
        self.new_title = faker.name()
        self.new_content = faker.text()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from xinge_push import (Message, Style, XingeApp, MessageIOS, TimeInterval,
                        ClickAction)

from notification_service.xinge import result_status
from notification_service.xinge.constant import *

import sys
reload(sys)
sys.setdefaultencoding('utf-8')


def _validate_push_params(accessId, secretKey, token, client_type, content,
                          title, msg_type, custom_info, environment):
    """
    对推送方法的参数进行校验
    :return: (error_code:int, error_msg:unicode)
        error_code 为 0，是成功
    """
    if accessId < 0 or not isinstance(accessId, int):
        return result_status.ACCESS_ID_ILLEGAL
    if not isinstance(secretKey, unicode):
        return result_status.SECRET_KEY_TYPE_ERROR
    if not isinstance(token, unicode):
        return result_status.TOKEN_TYPE_ERROR
    if client_type not in (
            CLIENT_TYPE_IOS,
            CLIENT_TYPE_ANDROID):
        return result_status.CLIENT_TYPE_ILLEGAL
    if not isinstance(content, unicode):
        return result_status.CONTENT_TYPE_ERROR
    if not isinstance(title, unicode):

        return result_status.TITLE_TYPE_ERROR
    if msg_type not in (
            MSG_TYPE_NOTIFICATION,
            MSG_TYPE_PENETRATE):
        return result_status.MSG_TYPE_ILLEGAL
    if not isinstance(custom_info, dict):
        return result_status.CUSTOM_INFO_TYPE_ERROR
    if environment not in (
            ENVIRONMENT_ANDROID,
            ENVIRONMENT_IOS_PRO,
            ENVIRONMENT_IOS_DEV):
        return result_status.ENVIRONMENT_ILLEGAL
    return result_status.SUCCESS


def _create_accept_time(time_plans):
    accept_time = []
    for plan in time_plans:
        if 'start' in plan and 'end' in plan:
            try:
                accept_time.append(TimeInterval(
                    int(plan['start']['hour']),
                    int(plan['start']['min']),
                    int(plan['end']['hour']),
                    int(plan['end']['min']),
                ))
            except:
                pass
    return accept_time


def _create_message_android(content, title, msg_type, custom_info):
    """
    生成需要发送的 Android 推送消息体
    :param content: 推送的主体内容
    :param title: 推送的标题
    :param custom_info: 推送定制信息
    :return: Message
    """
    msg = Message()
    msg.content = content
    msg.title = title
    msg.type = msg_type

    def _create_style(custom_info):
        """
        根据传入的字典创建 Style
        :param custom_info: 自定义信息
        :return: Style
        """
        style = Style()
        fields = {
            'title': 'title',
            'content': 'content',
            'builderId': 'builder_id',
            'ring': 'ring',
            'vibrate': 'vibrate',
            'clearable': 'clearable',
            'nId': 'n_id',
            'ringRaw': 'ring_raw',
            'lights': 'lights',
            'iconType': 'icon_type',
            'iconRes': 'icon_res',
            'styleId': 'style_id',
            'smallIcon': 'small_icon',
        }
        for key, value in fields.items():
            if value in custom_info:
                setattr(style, key, custom_info[value])
        return style

    msg.style = _create_style(custom_info)
    msg.acceptTime = _create_accept_time(custom_info.get('accept_time', ()))

    def _create_click_action(action):
        """
        根据传入的字典生成点击动作 action
        :param action: 动作的信息
        :return: ClickAction
        """
        if action == {} or 'action_type' not in action:
            return None
        click_action = ClickAction()
        click_action.actionType = action['action_type']
        if click_action.actionType == 1:
            click_action.activity = action.get('activity', '')
            click_action.intentFlag = action.get('aty_attr', {}).get('if', 0)
            click_action.pendingFlag = action.get('aty_attr', {}).get('pf', 0)
        elif click_action.actionType == 2:
            click_action.url = action.get('browser', {}).get('url', '')
            click_action.confirmOnUrl.get('browser', {}).get('confirm', 0)
        elif click_action.actionType == 3:
            click_action.intent = action.get('intent', '')
        else:
            raise ValueError('xinge push custom info action_type is illegal.')
        return click_action
    msg.action = _create_click_action(custom_info.get('action', {}))
    return msg


def _create_message_iOS(content, title, custom_info):
    """
    生成需要发送的 iOS 推送消息体

    信息体包含三种主要内容：
        1. APNs 相关的消息描述;
        2. 允许推送时间区间定制；
        3. 用户自定义键值对信息；
    :param content: 推送的主体内容
    :param title: 推送的标题
    :param custom_info: 推送定制信息
    :return: MessageIOS
    """
    msg = MessageIOS()
    aps = custom_info.pop('aps', {})
    alert = aps.get('alert', {})
    if 'title' not in alert:
        alert['title'] = title
    if 'body' not in alert:
        alert['body'] = content
    msg.alert = alert
    if 'badge' in aps:
        msg.badge = aps['badge']
    if 'category' in aps:
        msg.category = aps['category']
    msg.sound = aps.get('sound', 'default')

    msg.acceptTime = _create_accept_time(custom_info.pop('accept_time', ()))

    msg.custom = custom_info
    return msg


def push_single_device(accessId, secretKey, token, client_type, content,
                       title='通知', msg_type=MSG_TYPE_NOTIFICATION,
                       custom_info={}, environment=ENVIRONMENT_ANDROID):
    """
    对单一设备进行推送
    :param accessId: 等同于推送数据模型中 Server 的 app_id。
    :param secretKey: 等同于推送数据模型中 Server 的 app_secret
    :param token: 等同于推送数据模型中 Client 的 cid, 确定唯一设备
    :param client_type: 客户端类型，可选值为（xinge-ios, xinge-android）
    :param content: 推送的主体内容
    :param title: 推送标题
    :param msg_type: 推送类型,可选值（通知:1 ， 透传: 2）
    :param custom_info: 自定义通知参数
    :param environment: 向 iOS 设备推送时必填,1 表示推送生产环境;2 表示推送开发环境。
        推送 Android 平台不填或填 0。
    :return: (error_code:int, error_msg:unicode)
        error_code 为 0，是成功
    """
    # 校验数据
    try:
        accessId = int(accessId)
    except ValueError:
        return result_status.ACCESS_ID_NOT_NUMBER
    if client_type == CLIENT_TYPE_ANDROID:
        environment = ENVIRONMENT_ANDROID
    validate_result = _validate_push_params(accessId, secretKey, token,
                                            client_type, content, title,
                                            msg_type, custom_info, environment)
    if validate_result != result_status.SUCCESS:
        return validate_result

    # 构造推送信息体
    if client_type == CLIENT_TYPE_ANDROID:
        message = _create_message_android(content, title, msg_type, custom_info)
    elif client_type == CLIENT_TYPE_IOS:
        message = _create_message_iOS(content, title, custom_info)

    # 推送
    app = XingeApp(accessId, secretKey)
    push_result = app.PushSingleDevice(token, message, environment)
    return push_result


def check_authorization(accessId, secretKey):
    """
    检查 accessId 和 secretKey 是否能够通过验证
    :param accessId:
    :param secretKey:
    :return:
    """
    try:
        accessId = int(accessId)
    except ValueError:
        return False
    app = XingeApp(accessId, secretKey)
    try:
        (code, msg, _) = app.QueryDeviceCount()
    except ValueError:
        # 查询结果 SDK 无法加载为 JSON
        return True
    if code != 0:
        return False
    return True


__all__ = ['push_single_device', 'check_authorization']


if __name__ == '__main__':
    accessId = u'2100311092'
    secretKey = u'8d9bc98ffcdb833b369ae8ea72f8082c'
    token = u'4ebaca99000ec87349e64a5dbef6edbc19bb03a2'
    client_type = CLIENT_TYPE_ANDROID
    content = u'测试信鸽'
    title = u'通知标题'
    # result = push_single_device(accessId, secretKey, token, client_type, content, title)

    result = check_authorization(int(accessId), secretKey)
    print(result)


# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import sys

import xinge_push

from notification_service.push.base import (Sender, Message, ClientSystem,
                                            ServiceProvider, PushType, Environment)
from notification_service.exceptions import ClientTypeError
from notification_service.result import PushResult
from notification_service.states import (
    UnknowPushState, PushExcetion, Success)
from notification_service.utils import update_sub_dict

reload(sys)
sys.setdefaultencoding('utf-8')


class XingeMessage(Message):

    def _to_msg_object_iOS(self):
        msg = xinge_push.MessageIOS()
        msg.raw = {
            'aps': {
                'alert': {
                    'title': self.title,
                    'body': self.content,
                }
            }
        }
        return self._apply_push_type(self._apply_custom_msg(msg))

    def _to_msg_object_android(self):
        msg = xinge_push.Message()
        msg.raw = {
            'title': self.title,
            'content': self.content,
        }
        return self._apply_push_type(self._apply_custom_msg(msg))

    def _apply_custom_msg(self, msg):
        if self.custom_msg:
            msg.raw = update_sub_dict(msg.raw, self.custom_msg)
        return msg

    def _apply_push_type_iOS(self, msg):
        if self.push_type == PushType.custom_message:
            msg.raw['aps']['content-available'] = 1
            msg.raw['aps'].pop('alert', None)
        return msg

    def _apply_push_type_android(self, msg):
        if self.push_type == PushType.notification:
            style = xinge_push.Style()
            msg.raw['builder_id'] = style.builderId
            msg.raw['ring'] = style.ring
            msg.raw['vibrate'] = style.vibrate
            msg.raw['clearable'] = style.clearable
            msg.raw['n_id'] = style.nId
            msg.raw['ring_raw'] = style.ringRaw
            msg.raw['lights'] = style.lights
            msg.raw['icon_type'] = style.iconType
            msg.raw['icon_res'] = style.iconRes
            msg.raw['style_id'] = style.styleId
            msg.raw['small_icon'] = style.smallIcon
        return msg

    @property
    def environment_value(self):
        if self.client_system is ClientSystem.Android:
            return 0
        elif self.client_system is ClientSystem.iOS:
            if self.environment is Environment.production:
                return 1
            elif self.push_type is Environment.development:
                return 2


class XingeSender(Sender):

    def _init_config(self):
        super(XingeSender, self)._init_config()
        self._client = xinge_push.XingeApp(self.access_id, self.secret_key)

    def _verify_info(self):
        super(XingeSender, self)._verify_info()
        if self.service_provider != ServiceProvider.xinge:
            raise ClientTypeError(
                'service provider `{}` is not `xinge`'.format(self.service_provider))

    def check_authorization(self):
        try:
            response = self._client.QueryDeviceCount()
        except ValueError, e:
            # 查询结果 SDK 无法加载为 JSON
            return PushResult(Success(e))
        except Exception, e:
            return PushResult(PushExcetion(e))
        if response[0] != 0:
            return PushResult(UnknowPushState(response))
        else:
            return PushResult(Success(response))

    def push_single_device(self, msg, target):
        try:
            response = self._client.PushSingleDevice(
                target, msg.to_msg_object(), msg.environment_value)
        except Exception, e:
            return PushResult(PushExcetion(e))
        if response[0] != 0:
            return PushResult(UnknowPushState(response))
        else:
            return PushResult(Success(response))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
import collections


def from_ARN_to_region_name(arn):
    return arn.split(':')[3]


def update_sub_dict(d, u):
    """Update a dict from another dict, and include their sub dict."""
    for k, v in u.iteritems():
        if isinstance(v, collections.Mapping):
            d[k] = update_sub_dict(d.get(k, {}), v)
        else:
            d[k] = v
    return d



from .core import Device 

from .entity import (
    add,
    remove,
    device_tag,
    device_alias,
    device_regid,
)

__all__ = [
    Device,
    add,
    remove,
    device_tag,
    device_alias,
    device_regid,
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import


class StateBase(object):
    """Base class of states."""
    detail = ''

    def __init__(self, detail=''):
        self.detail = str(detail)

    @property
    def is_success(self):
        return isinstance(self, Success)

    @property
    def name(self):
        return self.__class__.__name__

    def __str__(self):
        return 'State name: {name}. Detail: {detail}.'\
            .format(name=self.name, detail=self.detail)

    def __unicode__(self):
        return unicode(self.__str__())


class PushState(StateBase):
    """Push state."""


class Success(StateBase):
    """Success."""


class UnknowPushState(PushState):
    pass


class PushExcetion(PushState):

    def __init__(self, exception, detail=''):
        detail += '{}: {}'.format(exception.__class__.__name__, str(exception))
        super(PushExcetion, self).__init__(detail=detail)


class CheckAuthorizationExcetion(PushExcetion):
    pass



# encoding: utf-8
from Channel import *


class Baidu_Push:

    def __init__(self, apiKey, secretKey):
        self.apiKey = apiKey
        self.secretKey = secretKey

    def push_single_device(self, channel_id, content, title="通知", type=1, client_type="baidu-android", deploy_status=2, baidu_obj={}):
        c = Channel(self.apiKey, self.secretKey)
        data = {}
        msg_type = 1 if type == 1 else 0
        data["msg_type"] = msg_type
        data["channel_id"] = int(channel_id)
        if client_type == 'baidu-android':
            message = {
                "title": title,
                "description": content
            }
            if baidu_obj:
                notification_builder_id = baidu_obj.get('notification_builder_id')
                if notification_builder_id is not None:
                    message['notification_builder_id'] = notification_builder_id
                notification_basic_style = baidu_obj.get('notification_basic_style')
                if notification_basic_style is not None:
                    message['notification_basic_style'] = notification_basic_style
                open_type = baidu_obj.get('open_type')
                if open_type is not None:
                    message['open_type'] = open_type
                url = baidu_obj.get('url')
                if url is not None:
                    message['url'] = url
                pkg_content = baidu_obj.get('pkg_content')
                if pkg_content is not None:
                    message['pkg_content'] = pkg_content
        elif client_type == 'baidu-ios':
            message = {
                "title": title,
                "description": content,
                "aps": {
                    "sound": "happy.caf",
                    "content-available": True
                }
            }
            aps = baidu_obj.get('aps')
            if aps:
                sound = aps.get('sound')
                badge = aps.get('badge')
                if badge is not None:
                    message['aps']['badge'] = badge
                if sound is not None:
                    message['aps']['sound'] = sound
            data['deploy_status'] = deploy_status
        data['msg'] = json.dumps(message)
        ret = c.pushSingle(data)
        if ret.get('error_code'):
            return {'error_msg': ret.get('error_msg')}
        return ret

    def fetchTag(self):
        '''
        获取百度推送标签，以验证app_secret是否正确
        '''
        c = Channel(self.apiKey, self.secretKey)
        tag = c.fetchTag()
        if tag.get('error_msg'):
            return {"errmsg": tag.get('error_msg'), "error_code": tag.get('error_code')}
        return {}



import json
import logging


BASE_URL = "https://api.jpush.cn/"
PUSH_URL = BASE_URL + 'v3/push'
VALIDATE_PUSH_URL = BASE_URL + 'v3/push/validate'

DEVICE_BASEURL = "https://device.jpush.cn/"
DEVICE_URL = DEVICE_BASEURL + "v3/devices/"
TAG_URL = DEVICE_BASEURL + "v3/tags/"
TAGLIST_URL = TAG_URL
ALIAS_URL = DEVICE_BASEURL + "v3/aliases/"

logger = logging.getLogger('jpush')

class Unauthorized(Exception):
    def from_response(cls, response):
        """Raised when we get a 401 from the server"""
        return cls("no authorization", "5000", "", response, 401,
            response.content)


class JPushFailure(Exception):
    """Raised when we get an error response from the server.
    :param args: For backwards compatibility, ``*args`` includes the status and
        response body.
    """

    error = None
    error_code = None
    details = None
    response = None

    def __init__(self, error, error_code, details, response, *args):
        self.error = error
        self.error_code = error_code
        self.details = details
        self.response = response
        super(JPushFailure, self).__init__(*args)

    @classmethod
    def from_response(cls, response):
        """Instantiate a ValidationFailure from a Response object"""

        try:
            payload = response.json()
            error = payload.get('error')
            error_code = error.get('code')
            details = error.get('message')
        except ValueError:
            error = response.reason
            error_code = None
            details = response.content

        logger.error(
            "Request failed with status %d: '%s %s': %s",
            response.status_code, error_code, error, json.dumps(details))

        return cls(error, error_code, details, response, response.status_code,
            response.content)



import json
import logging
import warnings

import requests

from . import common
from .push import Push
from .device import Device

logger = logging.getLogger('jpush')

class JPush(object):

    def __init__(self, key, secret):
        self.key = key
        self.secret = secret

        self.session = requests.Session()
        self.session.auth = (key, secret)

    def _request(self, method, body, url, content_type=None,
            version=None, params=None):

        headers = {}
        headers['user-agent'] = 'jpush-api-python-client'
        headers['connection'] = 'keep-alive'
        headers['content-type'] = 'application/json;charset:utf-8'

        logger.debug("Making %s request to %s. Headers:\n\t%s\nBody:\n\t%s",
            method, url, '\n\t'.join(
                '%s: %s' % (key, value) for (key, value) in headers.items()),
            body)

        response = self.session.request(
            method, url, data=body, params=params, headers=headers)

        logger.debug("Received %s response. Headers:\n\t%s\nBody:\n\t%s",
            response.status_code, '\n\t'.join(
                '%s: %s' % (key, value) for (key, value)
                in response.headers.items()),
            response.content)

        # if response.status_code == 401:
        #     raise common.Unauthorized.from_response(response)
        # elif not (200 <= response.status_code < 300):
        #     raise common.JPushFailure.from_response(response)

        return response

    def push(self, payload):
        """Push this payload to the specified recipients.

        Payload: a dictionary the contents to send, e.g.:
            {'aps': {'alert': 'Hello'}, 'android': {'alert': 'Hello'}}
        """
        warnings.warn(
            "JPush.push() is deprecated. See documentation on upgrading.",
            DeprecationWarning)
        body = json.dumps(payload)
        self._request('POST', body, common.PUSH_URL,
            'application/json', version=1)

    def create_push(self):
        """Create a Push notification."""
        return Push(self)

    def create_device(self):
        """Create a Device information."""
        return Device(self)



import re

# Value selectors; aliases, tags, etc.

def tag(*tags):
    """Select a (list of) tag(s)."""
    vtag = [t for t in tags]
    return {"tag": vtag}

def tag_and(*tag_ands):
    """Select a (list of) tag_and(s)."""
    vtag_and = [t for t in tag_ands]
    return {"tag_and": vtag_and}

def alias(*alias):
    """Select a (list of) alias(es)."""
    valias = [t for t in alias]
    return {"alias": valias}

def registration_id(*reg_ids):
    """Select a (list of) registration_id(s)."""
    vregistration_id = [t for t in reg_ids]
    return {"registration_id": vregistration_id}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from enum import Enum

from notification_service.exceptions import ClientTypeError, RequireFieldsError

__all__ = (
    'ClientSystem', 'ServiceProvider', 'Sender', 'Message', 'PushType', 'Environment')


class MatchEnumMixin():

    @classmethod
    def match(cls, value):
        """Match a enum object by keyword.

        Arguments:
            value {str} -- A keyword include by enum name.

        Raises:
            KeyError -- Cannot find a suitable item.

        Returns:
            Enum -- A suitable enum object.
        """
        for enum_key in cls.__members__.iterkeys():
            if value.lower() in enum_key.lower():
                return cls[enum_key]
        raise KeyError(
            'Your key is `{}`. Cannot find a suitable item.'.format(value))


class ClientSystem(MatchEnumMixin, Enum):
    iOS = 'iOS'
    Android = 'Android'


class ServiceProvider(MatchEnumMixin, Enum):
    AWS = 'AWS'
    jiguang = 'jiguang'
    xinge = 'xinge'
    baidu = 'baidu'


class PushType(Enum):
    notification = 1
    custom_message = 2


class Environment(Enum):
    production = 'production'
    development = 'development'


class ClientInfoMixin():
    """Set up and provide target client infomation.

    Returns:
        str -- Original client_type string.
    """

    @property
    def client_type(self):
        return self._client_type

    @client_type.setter
    def client_type(self, value):
        service_provider, system = value.split('-')
        self.service_provider = ServiceProvider.match(service_provider)
        self.client_system = ClientSystem.match(system)
        self._client_type = value


class RequireFieldsCheckMixin():
    """Check fields `required_fields` is supplied."""

    def _checkrequired_fields(self):
        if not isinstance(self.required_fields, set):
            self.required_fields = set(self.required_fields)
        if not self.required_fields.issubset(self.__dict__.keys()):
            need_fields = self.required_fields - set(self.__dict__.keys())
            raise RequireFieldsError(need_fields)


class Sender(ClientInfoMixin, RequireFieldsCheckMixin, object):
    _client_type = None
    service_provider = None
    client_system = None

    access_id = ''
    secret_key = ''

    required_fields = ['access_id', 'secret_key', 'client_type']

    def __init__(self, *args, **kwargs):
        """Init Sender

        Arguments:
            access_id {str} -- [description]
            secret_key {[type]} -- [description]
            client_type {[type]} -- [description]
        """
        self.update(*args, **kwargs)
        self._checkrequired_fields()
        self._init_config()
        self._verify_info()

    def _init_config(self):
        try:
            self.client_type = self.__dict__['client_type']
        except KeyError:
            raise ClientTypeError('`client_type` should not be empty.')

    def _verify_info(self):
        pass

    def check_authorization(self):
        raise NotImplementedError

    def push_single_device(self, msg, target):
        raise NotImplementedError

    def update(self, *args, **kwargs):
        return self.__dict__.update(*args, **kwargs)


class iOSMessageMixin():

    @property
    def environment(self):
        return self._environment

    @environment.setter
    def environment(self, value):
        self._environment = value


class Message(ClientInfoMixin, iOSMessageMixin, RequireFieldsCheckMixin, object):
    """Message ready to send."""

    _client_type = None
    service_provider = None
    client_system = None

    title = ''
    content = ''
    push_type = PushType.notification
    custom_msg = {}
    _enviorment = None

    required_fields = ['client_type', ]

    def __init__(self, *args, **kwargs):
        """Init message

        Arguments:
            client_type {str} -- Client infomation. Like: jiguang-ios.
            title {str} -- Title of message.
            content {str} -- Content of message.
            push_type {PushType} -- Type of message. It would be `notification` 
                or `custom_message`.
            custom_msg {dict} -- Custom message.
            environment {Environment} -- Only iOS need this. Describe whether the
                environment in which the client is located is a production 
                environment or a development environment.
        """
        self.update(*args, **kwargs)
        self._checkrequired_fields()
        self._init_config()

    def _init_config(self):
        self.client_type = self.__dict__['client_type']
        if self.__dict__.get('custom_info'):
            self.custom_msg = self.__dict__['custom_msg']
        if self.client_system == ClientSystem.iOS:
            self.environment = self.__dict__['environment']

    def update(self, *args, **kwargs):
        return self.__dict__.update(*args, **kwargs)

    def to_msg_object(self):
        if self.client_system is ClientSystem.iOS:
            return self._to_msg_object_iOS()
        if self.client_system is ClientSystem.Android:
            return self._to_msg_object_android()

    def _to_msg_object_iOS(self):
        raise NotImplementedError

    def _to_msg_object_android(self):
        raise NotImplementedError

    def _apply_custom_msg(self, msg):
        raise NotImplementedError

    def _apply_push_type(self, msg):
        if self.push_type is PushType.custom_message:
            if self.client_system is ClientSystem.iOS:
                return self._apply_push_type_iOS(msg)
            if self.client_system is ClientSystem.Android:
                return self._apply_push_type_android(msg)
        return msg

    def _apply_push_type_iOS(self, msg):
        raise NotImplementedError

    def _apply_push_type_android(self, msg):
        raise NotImplementedError



#!/usr/bin/env python
#-*- coding:utf8 -*-
import sys

if 2 != sys.version_info[0]:
    unicode = str

def add(*types):
    """Select a (list of) to be added objects(s)

    >>> add("registrationid1", "registrationid2")
    {'add': ['registrationid1', 'registrationid2']}
    >>> add("tag1", "tag2")                         
    {'add': ['tag1', 'tag2']}
    >>> add("alias1", "alias2")   
    {'add': ['alias1', 'alias2']}
    """
    vadd = [v for v in types]
    return {"add": vadd}

def remove(*types):
    """Select a (list of) to be removed objects(s)

    >>> remove("registrationid1", "registrationid2")   
    {'remove': ['registrationid1', 'registrationid2']}
    >>> remove("tag1", "tag2")                              
    {'remove': ['tag1', 'tag2']}
    >>> remove("alias1", "alias2")                        
    {'remove': ['alias1', 'alias2']}
    """
    vremove = [v for v in types]
    return {"remove": vremove}

def device_tag(*types):
    """Get a tag object

    >>> device_tag("")
    {'tags': ''}
    >>> device_tag("tag1")
    {'tags': 'tag1'}
    >>> device_tag(add("tag1", "tag2"), remove("tag3", "tag4"))
    {'tags': {'add': ['tag1', 'tag2'], 'remove': ['tag3', 'tag4']}}
    """
    tag = {}
    if 1 == len(types) and isinstance(types[0], (str, unicode)):
        tag["tags"] = types[0]
        return tag
    tag["tags"] = {}
    for t in types:
        for key in t:
            if key not in ('add', 'remove'):
                raise ValueError("Invalid tag '%s'" % t)
            tag["tags"][key] = t[key]
    return tag

def device_alias(*types):
    """Get an alias object

    >>> device_alias("")
    {'alias': ''}
    >>> device_alias("alias1")
    {'alias': 'alias1'}
    >>> device_alias(add("alias1", "alias2"), remove("alias3", "alias4"))
    {'alias': {'add': ['alias1', 'alias2'], 'remove': ['alias3', 'alias4']}}
    """
    alias = {}
    if 1 == len(types) and isinstance(types[0], (str, unicode)):
        alias["alias"] = types[0]
        return alias 
    alias["alias"] = {}
    for t in types:
        for key in t:
            if key not in ('add', 'remove'):
                raise ValueError("Invalid alias '%s'" % t)
            alias["alias"][key] = t[key]
    return alias

def device_regid(*types):
    """Get a registration_id object

    >>> device_regid("")
    {'registration_ids': ''}
    >>> device_regid("registration_id1")
    {'registration_ids': 'registration_id1'}
    >>> device_regid(add("registration_id1", "registration_id2"), remove("registration_id3", "registration_id4"))
    {'registration_ids': {'add': ['registration_id1', 'registration_id2'], 'remove': ['registration_id3', 'registration_id4']}}
    """
    registration_id = {}
    if 1 == len(types) and isinstance(types[0], (str, unicode)):
        registration_id["registration_ids"] = types[0]
        return registration_id
    registration_id["registration_ids"] = {}
    for t in types:
        for key in t:
            if key not in ('add', 'remove'):
                raise ValueError("Invalid registration_id '%s'" % t)
            registration_id["registration_ids"][key] = t[key]
    return registration_id

if "__main__" == __name__:
    print (add("1", "2"))
    print (device_tag(add("a", "b"), remove('1', '2')))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
import re
import setuptools
import setuptools.command.test
from setuptools import setup, find_packages


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.match("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version('notification_service')

LONG_DESCRIPTION = open('README.rst').read()


# -*- Command: setup.py test -*-

class pytest(setuptools.command.test.test):
    user_options = [('pytest-args=', 'a', 'Arguments to pass to py.test')]

    def initialize_options(self):
        setuptools.command.test.test.initialize_options(self)
        self.pytest_args = []

    def run_tests(self):
        import pytest as _pytest
        sys.exit(_pytest.main(self.pytest_args))


setup(
    name="notification-common-service",
    version=version,
    description="Notifcation common service package",
    long_description=LONG_DESCRIPTION,
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords='django gizwits',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'requests==2.8.1',
        'xinge-push==1.1.8.3',
        'boto3==1.4.4',
        'jpush==3.3.5',
    ],
    cmdclass={'test': pytest},
)



from .core import Push

from .audience import (
    tag,
    tag_and,
    alias,
    registration_id,
)

from .payload import (
    android,
    ios,
    winphone,
    platform,
    notification,
    message,
    audience,
    options,
)

# Common selector for audience & platform 

all_ = "all"
"""Select all, to do a broadcast.

Used in both ``audience`` and ``platform``.
"""

__all__ = [
    all_,
    Push,
    tag,
    tag_and,
    alias,
    registration_id,
    notification,
    message,
    platform,
    audience,
    options,
]



import json
import logging

from notification_service.jpush import common

logger = logging.getLogger('jpush')

class Push(object):
    """A push notification. Set audience, message, etc, and send."""

    def __init__(self, jpush):
        self._jpush = jpush
        self.audience = None
        self.notification = None
        self.platform = None
        self.options = None
        self.message = None

    @property
    def payload(self):
        data = {
            "audience": self.audience,
            "platform": self.platform,
        }
        if (self.notification is None) and (self.message is None):
            raise ValueError("Notification and message cannot be both empty")
        if self.notification is not None:
            data['notification'] = self.notification
        if self.options is not None:
            data['options'] = self.options
        if self.message is not None:
            data['message'] = self.message
        return data

    def send(self):
        """Send the notification.

        :returns: :py:class:`PushResponse` object with ``push_ids`` and
            other response data.
        :raises JPushFailure: Request failed.
        :raises Unauthorized: Authentication failed.

        """
        body = json.dumps(self.payload)
        response = self._jpush._request('POST', body,
            common.PUSH_URL, 'application/json', version=3)

        # print (response.content)
        # return PushResponse(response)
        return response.json()

    def send_validate(self):
        """Send the notification to validate.

        :returns: :py:class:`PushResponse` object with ``push_ids`` and
            other response data.
        :raises JPushFailure: Request failed.
        :raises Unauthorized: Authentication failed.

        """
        body = json.dumps(self.payload)
        response = self._jpush._request('POST', body,
            common.VALIDATE_PUSH_URL, 'application/json', version=3)

        # print (response.content)
        return PushResponse(response)


class PushResponse(object):
    """Response to a successful push notification send.

    Right now this is a fairly simple wrapper around the json payload response,
    but making it an object gives us some flexibility to add functionality
    later.

    """
    payload = None

    def __init__(self, response):
        data = response.json()
        self.payload = data

    def __str__(self):
        return self.payload
